#include "para.h"

long unsigned tv2ms(const struct timeval *tv)
{
	return tv->tv_sec * 1000 + (tv->tv_usec + 500)/ 1000;
}

void ms2tv(const long unsigned n, struct timeval *tv)
{
	tv->tv_sec = n / 1000;
	tv->tv_usec = (n % 1000) * 1000;
}

/*
 * If b > a, compute b - a and return 1. if b < a compute a - b and return -1
 */
int tv_diff(const struct timeval *b, const struct timeval *a, struct timeval *diff)
{
	int ret = 1;

	if ((b->tv_sec < a->tv_sec) ||
		((b->tv_sec == a->tv_sec) && (b->tv_usec < a->tv_usec))) {
		const struct timeval *tmp = a;
		a = b;
		b = tmp;
		ret = -1;
	}
	if (!diff)
		return ret;
	diff->tv_sec = b->tv_sec - a->tv_sec;
	if (b->tv_usec < a->tv_usec) {
		diff->tv_sec--;
		diff->tv_usec = 1000 * 1000 - a->tv_usec + b->tv_usec;
	} else
		diff->tv_usec = b->tv_usec - a->tv_usec;
	return ret;
}

/*
 * convert double to struct timeval
 */
void d2tv(double x, struct timeval *tv)
{
	tv->tv_sec = x;
	tv->tv_usec = (x - (double)tv->tv_sec) * 1000.0 * 1000.0 + 0.5;
}
/*
 * add two structs of type timeval
 */
void tv_add(const struct timeval *a, const struct timeval *b,
	struct timeval *sum)
{
	sum->tv_sec = a->tv_sec + b->tv_sec;
	if (a->tv_usec + b->tv_usec >= 1000 * 1000) {
		sum->tv_sec++;
		sum->tv_usec = a->tv_usec + b->tv_usec - 1000 * 1000;
	} else
		sum->tv_usec = a->tv_usec + b->tv_usec;
}

/*
 * compute integer multiple of given struct timeval
 */
void tv_scale(const unsigned long mult, const struct timeval *tv,
	struct timeval *result)
{
	result->tv_sec = mult * tv->tv_sec;
	result->tv_sec += tv->tv_usec * mult / 1000 / 1000;
	result->tv_usec = tv->tv_usec * mult % (1000 * 1000);
}

/*
 * compute fraction of given struct timeval
 */
void tv_divide(const unsigned long div, const struct timeval *tv,
	struct timeval *result)
{
	long unsigned q = tv->tv_usec / div;
	result->tv_sec = tv->tv_sec / div;
	result->tv_usec = (tv->tv_sec - result->tv_sec * div) * 1000 * 1000 / div;
	if (result->tv_usec + q >= 1000 * 1000) {
		result->tv_sec++;
		result->tv_usec = 1000 * 1000 - result->tv_usec - q;
	} else
		result->tv_usec += q;
}

/*
 * compute x := (a * t1 + b * t2) / (|a| + |b|). a, b may be negative.
 * returns sign(x) and stores |x| in the result pointer.
 */
int tv_convex_combination(const long a, const struct timeval *tv1,
		const long b, const struct timeval *tv2,
		struct timeval *result)
{
	struct timeval tmp1, tmp2, tmp3;
	int ret = 1, subtract = ((a > 0 && b < 0) || (a < 0 && b > 0));
	unsigned long a1 = ABS(a), b1 = ABS(b);

	tv_scale(a1, tv1, &tmp1);
	tv_scale(b1, tv2, &tmp2);
	if (subtract)
		ret = tv_diff(&tmp1, &tmp2, &tmp3);
	else
		tv_add(&tmp1, &tmp2, &tmp3);
	if (a1 + b1)
		tv_divide(a1 + b1, &tmp3, result);
	else {
		result->tv_sec = 0;
		result->tv_usec = 0;
	}
	if (!a || !b) {
		if (a + b < 0)
			ret = -1;
	} else
		if (a < 0)
			ret = -ret;
	return ret;
}
